@@ -57,9 +57,10 @@ setMethod(
     {
         ngroups <- object@Data$ngroups
         Theta <- object@Model$Theta
+        colnames(Theta) <- extract.mirt(object, 'factorNames')[1:ncol(Theta)]
         ret <- vector('list', ngroups)
         items <- vector('list', object@Data$nitems + 1L)
-        names(items) <- c(colnames(object@Data$data), 'Class.Proportions')
+        names(items) <- c(colnames(object@Data$data), 'Class.Probability')
         for(g in 1L:ngroups){
             ret[[g]] <- items
             pars <- object@ParObjects$pars[[g]]
@@ -70,8 +71,10 @@ setMethod(
                 rownames(P) <- paste0('Class_', 1L:nrow(P))
                 ret[[g]][[i]] <- P
             }
-            ret[[g]][[i+1L]] <- round(object@Internals$Prior[[g]], digits)
+            ret[[g]][[i+1L]] <- data.frame(Theta, prob=round(object@Internals$Prior[[g]], digits))
+            rownames(ret[[g]][[i+1L]]) <- paste0('Class_', 1L:nrow(ret[[g]][[i+1L]]))
         }
+        names(ret) <- extract.mirt(object, 'groupNames')
         if(length(ret) == 1L) ret <- ret[[1L]]
         ret
     }
@@ -81,7 +84,7 @@ setMethod(
     signature = 'DiscreteClass',
     definition = function(object, drop = TRUE, ...){
         class(object) <- 'MultipleGroupClass'
-        ret <- coef(object, ...)
+        ret <- coef(object, discrete = TRUE, ...)
         if(drop)
             if(length(ret) == 1L) ret <- ret[[1L]]
         ret
@@ -120,6 +123,9 @@ setMethod(
                                               strip.border = list(col = "black")),
                           auto.key = list(space = 'right'), ...)
     {
+        if(extract.mirt(x, 'ngroups') > 1L)
+            stop('plot methods do not support multiple group latent class models yet',
+                 call.=FALSE)
         so <- summary(x)
         index <- which.items
         names <- colnames(x@Data$data)
