@@ -6,7 +6,9 @@
 #' For Rasch, partial credit, and rating scale models infit and outfit statistics are
 #' also produced. Poorly fitting items should be inspected with the empirical plots/tables
 #' for unidimensional models, otherwise \code{\link{itemGAM}} can be used to diagnose
-#' where the functional form of the IRT model was misspecified.
+#' where the functional form of the IRT model was misspecified, or models can be refit using
+#' more flexible semi-parametric response models (e.g., \code{itemtype = 'spline'}).
+#' For discrete models, only the S-X2 statistic will be calculated.
 #'
 #' @aliases itemfit
 #' @param x a computed model object of class \code{SingleGroupClass},
@@ -126,6 +128,15 @@
 #' Theta <- fscores(raschfit, method = 'ML')
 #' itemfit(raschfit, Theta=Theta, infit = TRUE)
 #'
+#' # fit a new more flexible model for the mis-fitting item
+#' itemtype <- c(rep('2PL', 20), 'spline')
+#' x2 <- mirt(data, 1, itemtype=itemtype)
+#' itemfit(x2)
+#' itemplot(x2, 21)
+#' anova(x2, x)
+#'
+#' #------------------------------------------------------------
+#'
 #' #similar example to Kang and Chen 2007
 #' a <- matrix(c(.8,.4,.7, .8, .4, .7, 1, 1, 1, 1))
 #' d <- matrix(rep(c(2.0,0.0,-1,-1.5),10), ncol=4, byrow=TRUE)
@@ -181,7 +192,7 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
 
     fn <- function(ind, Theta, obj, vals, ...){
         tmpobj <- obj
-        tmpdat <- imputeMissing(obj, Theta[[ind]])
+        tmpdat <- imputeMissing(obj, Theta[[ind]], warn=FALSE)
         tmpmod <- mirt(tmpdat, model=1, TOL=NA,
                        technical=list(customK=obj@Data$K, message=FALSE, warn=FALSE))
         tmpobj@Data <- tmpmod@Data
@@ -192,13 +203,6 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
     if(missing(x)) missingMsg('x')
     if(is(x, 'MixedClass'))
         stop('MixedClass objects are not supported', call.=FALSE)
-    discrete <- FALSE
-    if(is(x, 'DiscreteClass')){
-        if(is.null(Theta))
-            Theta <- fscores(x, method=method, full.scores=TRUE, plausible.draws=impute, ...)
-        class(x) <- 'MultipleGroupClass'
-        discrete <- TRUE
-    }
     if(!is.null(empirical.plot) && !is.null(empirical.table))
         stop('Please select empirical.plot or empirical.table, not both', call.=FALSE)
     if(!is.null(empirical.plot))
@@ -221,6 +225,32 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
     if(any(is.na(x@Data$data)) && (Zh || S_X2 || infit) && impute == 0)
         stop('Only X2 and G2 can be computed with missing data. Consider using imputed datasets', call.=FALSE)
 
+    if(is(x, 'MultipleGroupClass') || is(x, 'DiscreteClass')){
+        discrete <- is(x, 'DiscreteClass')
+        ret <- vector('list', x@Data$ngroups)
+        if(is.null(Theta))
+            Theta <- fscores(x, method=method, full.scores=TRUE, plausible.draws=impute, ...)
+        for(g in 1L:x@Data$ngroups){
+            if(impute > 0L){
+                tmpTheta <- vector('list', impute)
+                for(i in 1L:length(tmpTheta))
+                    tmpTheta[[i]] <- Theta[[i]][x@Data$groupNames[g] == x@Data$group, , drop=FALSE]
+            } else tmpTheta <- Theta[x@Data$groupNames[g] == x@Data$group, , drop=FALSE]
+            tmp_obj <- MGC2SC(x, g)
+            ret[[g]] <- itemfit(tmp_obj, Zh=Zh, X2=X2, group.size=group.size, group.bins=group.bins,
+                                group.fun=group.fun, mincell=mincell, mincell.X2=mincell.X2, infit=infit,
+                                S_X2.tables=S_X2.tables, empirical.plot=empirical.plot,
+                                empirical.table=empirical.table, G2=G2,
+                                Theta=tmpTheta, empirical.CI=empirical.CI, method=method,
+                                impute=impute, discrete=discrete, digits=digits, S_X2=S_X2, ...)
+        }
+        names(ret) <- x@Data$groupNames
+        if(extract.mirt(x, 'ngroups') == 1L) return(ret[[1L]])
+        return(ret)
+    }
+    dots <- list(...)
+    discrete <- dots$discrete
+    discrete <- ifelse(is.null(discrete), FALSE, discrete)
     if(impute != 0 && !is(x, 'MultipleGroupClass')){
         if(impute == 0)
             stop('Fit statistics cannot be computed when there are missing data. Pass a suitable
@@ -229,7 +259,8 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
         if(sum(is.na(x@Data$data)) / length(x@Data$data) > .10)
             warning('Imputations for large amounts of missing data may be overly conservative', call.=FALSE)
         stopifnot(impute > 1L)
-        Theta <- fscores(x, plausible.draws = impute, method = ifelse(method == 'MAP', 'MAP', 'EAP'), ...)
+        if(is.null(Theta))
+            Theta <- fscores(x, plausible.draws = impute, method = ifelse(method == 'MAP', 'MAP', 'EAP'), ...)
         collect <- vector('list', impute)
         vals <- mod2values(x)
         vals$est <- FALSE
@@ -255,30 +286,6 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
         ret[,sapply(ret, class) == 'numeric'] <- round(ret[,sapply(ret, class) == 'numeric'], digits)
         return(ret)
     }
-    if(is(x, 'MultipleGroupClass')){
-        ret <- vector('list', x@Data$ngroups)
-        if(is.null(Theta))
-            Theta <- fscores(x, method=method, full.scores=TRUE, plausible.draws=impute, ...)
-        for(g in 1L:x@Data$ngroups){
-            if(impute > 0L){
-                tmpTheta <- vector('list', impute)
-                for(i in 1L:length(tmpTheta))
-                    tmpTheta[[i]] <- Theta[[i]][x@Data$groupNames[g] == x@Data$group, , drop=FALSE]
-            } else tmpTheta <- Theta[x@Data$groupNames[g] == x@Data$group, , drop=FALSE]
-            tmp_obj <- MGC2SC(x, g)
-            ret[[g]] <- itemfit(tmp_obj, Zh=Zh, X2=X2, group.size=group.size, group.bins=group.bins,
-                                group.fun=group.fun, mincell=mincell, mincell.X2=mincell.X2, infit=infit,
-                                S_X2.tables=S_X2.tables, empirical.plot=empirical.plot,
-                                empirical.table=empirical.table, G2=G2,
-                                Theta=tmpTheta, empirical.CI=empirical.CI, method=method,
-                                impute=impute, discrete=discrete, digits=digits, S_X2=S_X2, ...)
-        }
-        names(ret) <- x@Data$groupNames
-        return(ret)
-    }
-    dots <- list(...)
-    discrete <- dots$discrete
-    discrete <- ifelse(is.null(discrete), FALSE, discrete)
     if(S_X2.tables || discrete) Zh <- X2 <- FALSE
     ret <- data.frame(item=colnames(x@Data$data)[which.items])
     J <- ncol(x@Data$data)
@@ -528,7 +535,7 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
         quadpts <- dots$quadpts
         if(is.null(quadpts) && QMC) quadpts <- 15000L
         if(is.null(quadpts)) quadpts <- select_quadpts(x@Model$nfact)
-        if(x@Model$nfact > 3L && !QMC && method %in% c('EAP', 'EAPsum'))
+        if(x@Model$nfact > 3L && !QMC && method %in% c('EAP', 'EAPsum') && !discrete)
             warning('High-dimensional models should use quasi-Monte Carlo integration. Pass QMC=TRUE',
                     call.=FALSE)
         theta_lim <- dots$theta_lim
@@ -548,9 +555,10 @@ itemfit <- function(x, which.items = 1:extract.mirt(x, 'nitems'),
             S_X2[i] <- sum((O[[i]] - E[[i]])^2 / E[[i]], na.rm = TRUE)
             df.S_X2[i] <- sum(!is.na(E[[i]])) - nrow(E[[i]]) - sum(pars[[i]]@est)
         }
-        S_X2[df.S_X2 <= 0] <- NaN
-        ret$S_X2 <- na.omit(S_X2)
-        ret$df.S_X2 <- na.omit(df.S_X2)
+        df.S_X2[df.S_X2 < 0] <- 0
+        S_X2[df.S_X2 == 0] <- NaN
+        ret$S_X2 <- S_X2
+        ret$df.S_X2 <- df.S_X2
         ret$p.S_X2 <- 1 - suppressWarnings(pchisq(ret$S_X2, ret$df.S_X2))
     }
     ret[,sapply(ret, class) == 'numeric'] <- round(ret[,sapply(ret, class) == 'numeric'], digits)
