@@ -26,7 +26,7 @@ void P_nested(vector<double> &, const vector<double> &,
 void P_comp(vector<double> &, const vector<double> &,
     const NumericMatrix &, const int &, const int &);
 
-void P_lca(vector<double> &, const vector<double> &, const vector<double> &, 
+void P_lca(vector<double> &, const vector<double> &,
 	const NumericMatrix &, const int &, const int &, const int &, const int &);
 
 void _computeItemTrace(vector<double> &, const NumericMatrix &,
